/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2018 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/* MODULE main */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "clockMan1.h"
#include "Flash.h"
#include "pin_mux.h"
#include "dmaController1.h"
#if CPU_INIT_CONFIG
  #include "Init_Config.h"
#endif


volatile int exit_code = 0;
/* User includes (#include below this line is not maintained by Processor Expert) */

#include <stdint.h>
#include <stdbool.h>
#include <string.h>
/* Declare a FLASH config struct which initialized by FlashInit, and will be used by all flash operations */
flash_ssd_config_t flashSSDConfig;



uint8_t Write_ByteArray(uint8_t* source, uint32_t* target,uint16_t size)
{
	unsigned int cycles=size/4;
	unsigned int remain=size%4;
	unsigned int offset=0;
	for (int i=0;i<cycles;i++)
	{
		while ((FTFC-> FSTAT & FTFC_FSTAT_CCIF_MASK) == 0){}
		*(int*)(target+offset)=*(int*)(source+offset);
		offset+=4;
	}

	for (int i=0;i<remain;i++)
	{
			while ((FTFC-> FSTAT & FTFC_FSTAT_CCIF_MASK) == 0){}
			*(target+offset)=*(char*)(source+offset);
			offset+=1;
		}

	return 0;
}

uint8_t Read_ByteArray(uint8_t* source, uint32_t* target, uint16_t size)
{
	unsigned int cycles=size/4;
	unsigned int remain=size%4;
	unsigned int offset=0;
	for (int i=0;i<cycles;i++)
	{
		while ((FTFC-> FSTAT & FTFC_FSTAT_CCIF_MASK) == 0){}
		*(int*)(source+offset)=*(int*)(target+offset);
		offset+=4;
	}

	for (int i=0;i<remain;i++)
	{
			while ((FTFC-> FSTAT & FTFC_FSTAT_CCIF_MASK) == 0){}
			*(source+offset)=*(uint8_t*)(target+offset);
			offset+=1;
		}

	return 0;
}


int main(void)
{
  /* Write your local variable definition here */
  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  #ifdef PEX_RTOS_INIT
    PEX_RTOS_INIT();                   /* Initialization of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of Processor Expert internal initialization.                    ***/

  /* Write your code here */

    /* Initialize and configure clocks
     *  -   see clock manager component for details
     */
    CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
            g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_AGREEMENT);
    /* Disable cache to ensure that all flash operations will take effect instantly,
     * this is device dependent */
    PINS_DRV_Init(NUM_OF_CONFIGURED_PINS, g_pin_mux_InitConfigArr);







FLASH_DRV_Init(&Flash_InitConfig0, &flashSSDConfig);
LPUART_DRV_Init(INST_LPUART1, &lpuart1_State, &lpuart1_InitConfig0);


//uint8_t Array[25] = {0x01, 0xFF, 0x00, 0x00, 0x00, 0x64, 0x00, 0x64, 0x96, 0x5A, 0x1E, 0x31, 0x01, 0x00, 0x64, 0x04,0xA1,0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x06, 0x08};
uint8_t Array[25];
uint8_t SendArray[25];


/////// When I use this function, I can't nothing to read (before disconnection power 12V, after disconnection 12V ) ////
//FLASH_DRV_EEEWrite(&flashSSDConfig, flashSSDConfig.EERAMBase, sizeof(Array), Array);



    while (1)
   {
    	    	if (LPUART_DRV_ReceiveData(INST_LPUART1,Array,25U) == STATUS_SUCCESS && Array[0] == 0x01)
    	    		  {
    	    		Write_ByteArray(Array,&flashSSDConfig.EERAMBase,25);
    	    		OSIF_TimeDelay(1000);
    	    		  }

    	    	Read_ByteArray(SendArray,&flashSSDConfig.EERAMBase,25);

LPUART_DRV_SendData(INST_LPUART1,SendArray,25);
OSIF_TimeDelay(2000);
   }



  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;) {
    if(exit_code != 0) {
      break;
    }
  }
  return exit_code;
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
** @}
*/

/*!
  \brief Interrupt handler for Flash Command Complete event
*/


/*
** ###################################################################
**
**     This file was created by Processor Expert 10.1 [05.21]
**     for the Freescale S32K series of microcontrollers.
**
** ###################################################################
*/
